#include "stdafx.h"
/*****************************************************************************
    Copyright  1998, Joseph M. Newcomer. All Rights Reserved.

    Permission is granted to use this in conformance with the rules of
    www.codeguru.com.
*****************************************************************************/

/****************************************************************************
*                                getDriveType
* Inputs:
*       LPCTSTR fullpath: Full path to some destination
* Result: UINT
*       DRIVE_ value as returned by GetDriveType
* Effect: 
*       Properly splits up the input string and figures out how to call
*       GetDriveType 
* Notes:
*       Handles the special case of \\.\name\, and a path that includes
*       a file name
****************************************************************************/

UINT getDriveType(LPCTSTR fullpath)
    {
     TCHAR drv[_MAX_PATH];
     TCHAR path[_MAX_PATH];
     TCHAR filename[_MAX_PATH];
     TCHAR ext[_MAX_PATH];

     _splitpath(fullpath, drv, path, filename, ext);
     if(lstrlen(drv) != 0)
        { /* drv */
         // The following is required for Windows 95; the
         // GetDriveType returns a different value for c: and c:\
         //      Win95              WinNT
         // c:   DRIVE_NO_ROOT_DIR  DRIVE_FIXED
         // c:\  DRIVE_FIXED        DRIVE_FIXED
         lstrcat(drv, _T("\\"));
         return GetDriveType(drv);
        } /* drv */
     else
        { /* what else? */
         // If the name does not start with \, it is a relative path
         // use the current directory
         // If the name starts with \anythingbut\, it is a path rooted
         // in the current drive.

         if(path[0] != _T('\\') ||
            (lstrlen(path) > 2 && path[0] == _T('\\') &&
                path[1] != _T('\\')))
            return GetDriveType(NULL);

         // The trick here is that if the path starts with \\, we try 
         // GetDeviceType. If it returns DRIVE_NO_ROOT_DIR, we create a new 
         // name by dropping off the \\ sequence. We also handle the special
         // case of \\.\

         // If it does not start with \\, we return the result directly
         UINT result = GetDriveType(path);
         if(result == DRIVE_NO_ROOT_DIR &&
            lstrlen(path) >= 4 && path[0] == _T('\\') && path[1] == _T('\\')
                              && path[2] == _T('.') && path[3] == _T('\\'))
            { /* \\. */
             if(lstrlen(path) == 4) // Win95 and WinNT _splitpath parse differently!
                lstrcat(path, filename); // [\\.\] [c] => [\\.\c]
             LPTSTR p = &path[4]; //  \\.\c
                                  //      ^
                                  //      p

             LPTSTR b = strchr(p, _T('\\'));
             if(b != NULL)
                { /* logical drive */
                 // \\.\c\whatever
                 //     ^^
                 //     ||
                 //     |b
                 //     p
                 *b++ = _T(':');
                 *b++ = _T('\\'); // needed for Win95, see earlier comment
                 *b = _T('\0');
                 // \\.\c:*      where * = NUL
                } /* logical drive */
             else
                lstrcat(p, _T(":\\"));

             // Request the drive type for the newly computed name
             return GetDriveType(p);
            } /* \\. */

         if(lstrlen(path) < 2 || path[0] != _T('\\') || path[1] != _T('\\'))
            return result;

         return GetDriveType(path);
        } /* what else? */
            
    }
